package org.cyberlab;

import org.cyberlab.R;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;


/**
 * 
 * Custom button layout for the quiz. This layout holds text and two buttons which can be shown or hidden. There are two icons.
 * Depending of which you want to show call either wrong or correct. To reset button to default call reset. 
 * Rationale: Android does not offer buttons with text and icons. 
 * @author Mats-Gøran Karlsen
 *
 */
public class ImageButtonWithText extends RelativeLayout {
	
	public ImageButtonWithText(Context context, int id) {
		super(context);
		// if our context is not Activity we can't get View supplied by id
	    if (!(context instanceof Activity))
	        return;

	    // find relative layout by id  
	    View v = ((Activity)context).findViewById(id);
	     
	    // is it RelativeLayout ?
	    if (!(v instanceof RelativeLayout))
	        return;

	    //cast it to relative layout
	    RelativeLayout layout = (RelativeLayout)v;
	                 
	    // copy layout parameters 
	    ViewGroup.LayoutParams params = layout.getLayoutParams();
	    this.setLayoutParams(params);
	      
	    // here I am using temporary instance of Button class
	    // to get standard button background and to get button text color
	    Button bt = new Button(context);
	    this.setBackgroundDrawable(bt.getBackground());
	     
	    // copy all child from relative layout to this button
	    while (layout.getChildCount() > 0)
	    {
	         View vchild = layout.getChildAt(0);
	         layout.removeViewAt(0);
	         this.addView(vchild);
	          
	         // if child is textView set its color to standard buttong text colors
	         // using temporary instance of Button class
	         if (vchild instanceof TextView  )
	         {
	            ((TextView)vchild).setTextColor(bt.getTextColors());
	         }
	          
	         // just to be sure that child views can't be clicked and focused
	         vchild.setClickable(false);
	         vchild.setFocusable(false);
	         vchild.setFocusableInTouchMode(false);
	     }
	      
	     // remove all view from layout (maybe it's not necessary)
	     layout.removeAllViews();
	      
	     // set that this button is clickable, focusable, ...
	     this.setClickable(true);
	     this.setFocusable(true);
	     this.setFocusableInTouchMode(false);
	  
	     // replace relative layout in parent with this one modified to looks like button
	     ViewGroup vp = (ViewGroup)layout.getParent();
	     int index = vp.indexOfChild(layout);
	     vp.removeView(layout);
	     vp.addView(this,index);
	      
	     this.setId(id);
	}


	public void setClickable(boolean value){
		super.setClickable(value);
		if (value == false) {
			((TextView)findViewById(R.id.buttonText)).setTextColor(Color.GRAY);
		}else{
			((TextView)findViewById(R.id.buttonText)).setTextColor(Color.BLACK);
		}
	}

	/**
	 * Shows wrong image
	 */
	public void correct(){
		findViewById(R.id.imageCorrect).setVisibility(VISIBLE);
		findViewById(R.id.imageWrong).setVisibility(GONE);
	}

	/**
	 * Resets button to default: both images hidden
	 */
	public void reset(){
		findViewById(R.id.imageCorrect).setVisibility(GONE);
		findViewById(R.id.imageWrong).setVisibility(GONE);
		setClickable(true);
	}
	
	// method for setting texts for the text views
	public void setText(CharSequence text)
	{
		((TextView)findViewById(R.id.buttonText)).setText(text);
	
	}

	/**
	 * Shows correct image
	 */
	public void wrong(){
		findViewById(R.id.imageCorrect).setVisibility(GONE);
		findViewById(R.id.imageWrong).setVisibility(VISIBLE);
		setClickable(false);
	}

}
