package org.cyberlab;

import java.net.Authenticator;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cyberlab.local.activity.AbstractGeneralActivity;
import org.cyberlab.local.activity.task.CreateUnknownTask;
import org.cyberlab.local.activity.task.LicovMatchTask;
import org.cyberlab.animation.MyTranslateAnimation;
import org.cyberlab.local.recognition.LocalFeaturesExtractorFactory;
import org.cyberlab.local.recognition.LocalMatch;
import org.cyberlab.local.recognition.LocalModel;
import org.cyberlab.local.recognition.LICOV.LICOVMatch;
import org.cyberlab.support.FileStorage;
import org.cyberlab.support.ImageOps;
import org.cyberlab.support.UrlAuthentication;
import org.cyberlab.local.db.QuizSQLiteDBAdapter;

import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * 
 * The activity that is started when the application launches or a quiz category finishes. Automatically starts WelcomeActivity
 * on first app launch. Camera activity is automatically started each time this class starts. Options menu offer application settings.
 * @author Mats-Gøran Karlsen
 *
 */
public class InitializeCategoryActivity extends AbstractGeneralActivity<InitializeCategoryActivity> implements OnClickListener{

	private static final String TAG = "ObjectInitiatesSet";
	private static final String ASSET_FILE = "quizdb.sqlite";
	private static final String HTTP_URL_PROTOCOL = "http";
	private static final int WELCOME = 101;
	private SharedPreferences prefs;
	private Handler handler;
	private LinearLayout mainLayout;
	private RelativeLayout newCategorySlideLayout, allreadyAnsweredLayout;
	private TextView youChoseCategory, categoryDoneHeader, availableCategoriesList;
	private Animation slideIn, slideOut;
	private ImageButton imgButton;
	private ImageView bar;
	private MyTranslateAnimation scanbarTask;
	private CreateUnknownTask generateUnknown;
	private LicovMatchTask matchUnknown;
	private List<LocalMatch<LICOVMatch>> matches;
	private QuizSQLiteDBAdapter mDbAdapter = null;
	private MemoryHelper memoryhelper;
	private boolean working = false;
	
	@Override
	public void threadFinished(int id) {
		if (id == 1) {
			//continue with matching
			matchUnknown = new LicovMatchTask(this, 2, generateUnknown.getUnknown());
			LocalModel<?>[] localmodels = new LocalModel<?>[memoryhelper.getModels(this).getModelData().size()];
			for (int i = 0; i < localmodels.length; i++) {
				localmodels[i] = memoryhelper.getModels(this).getModelData().get(i);
			}
			matchUnknown.execute(localmodels);
			
		}
		if (id == 2) {
			//retrieve matches
			matches = matchUnknown.getMatches();
			scanbarTask.stopAnimation();
			checkMatch();
		}
	}
	
	@Override
	public void onClick(View v) {
		startCameraActivity();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.quizoptions, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.preferences:
			Intent intent = new Intent(InitializeCategoryActivity.this, QuizPrefActivity.class);
			startActivity(intent);
			return true;
		case R.id.download:
			if (mDbAdapter != null) {
				mDbAdapter.close();
			}
			
			fetchDatabase(true);
			fetchData(true);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	protected void cameraActivityResult() {
		working = true;
		Bitmap image = ImageOps.loadImage(  mCameraFile.getPath(), 
											Integer.parseInt( prefs.getString("imagewidth", null) ), 
											Integer.parseInt( prefs.getString("imageheight", null) ),
											true );
		imgButton.setImageBitmap(image);
		imgButton.setClickable(false);
		//scanbarTask = new ScanAnimationTask(InitializeCategoryActivity.this, bar);
		
		scanbarTask = new MyTranslateAnimation(bar, Animation.RELATIVE_TO_PARENT, 0, Animation.RELATIVE_TO_PARENT, 0, 
				Animation.RELATIVE_TO_PARENT, 0, Animation.RELATIVE_TO_PARENT, 1.0f);
		generateUnknown = new CreateUnknownTask(this, 1, 
				LocalFeaturesExtractorFactory.socketExtractor(  prefs.getString("detectorhost", null), 
																Integer.parseInt( prefs.getString("detectorport", null) )));
		generateUnknown.execute(image);
		scanbarTask.startAnimation();
	}

	@Override
	protected void libraryActivityResult() {/*not used*/}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == WELCOME) {
			startCameraActivity();
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
        if (savedInstanceState != null && savedInstanceState.containsKey("working")) {
			working = savedInstanceState.getBoolean("working");
		}
		setContentView(R.layout.initcategory);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		setTitle("Find a category");
		prefs = PreferenceManager.getDefaultSharedPreferences(this);
        PreferenceManager.setDefaultValues(this, R.xml.preference, false);
        if (prefs.contains("welcomerunonce") && !working) {
        	/* 
        	 * Start camera as early as possible so main thread can perform as many operations as possible
	  		 * in the background to give the impression of faster computation
	  		 * 
	  		 */
        	startCameraActivity();
		}
        handler = new Handler();
		//initialize main layout
		mainLayout = (LinearLayout) findViewById(R.id.mainLayout);
		mainLayout.setVisibility(LinearLayout.VISIBLE);
		imgButton = (ImageButton) findViewById(R.id.resultButton);
		imgButton.setOnClickListener(this);
		imgButton.setClickable(true);
		bar = (ImageView) findViewById(R.id.scanbar);
		//initialize slide layout
		newCategorySlideLayout = (RelativeLayout) findViewById(R.id.newCategorySliderLayout);
		youChoseCategory = (TextView) findViewById(R.id.chosenCategoryText);
		//initialize already done layout
		allreadyAnsweredLayout = (RelativeLayout) findViewById(R.id.answeredLayout);
		categoryDoneHeader = (TextView) findViewById(R.id.categoryDoneHeader);
		availableCategoriesList = (TextView) findViewById(R.id.availableCategoriesList);
		//initialize animations
        slideIn = AnimationUtils.loadAnimation(this, R.anim.slidein);
        slideOut = AnimationUtils.loadAnimation(this, R.anim.slideout);
        //initialize database and memory helper
        fetchDatabase(false);
		fetchData(false);
		if (!prefs.contains("welcomerunonce")) {
  			startActivityForResult(new Intent(InitializeCategoryActivity.this, WelcomeActivity.class), WELCOME);
		}
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		if (scanbarTask != null && !scanbarTask.isAnimating()) {
			imgButton.setClickable(true);
		}
		
	}

	@Override
	protected void onStart() {
		// TODO Auto-generated method stub
		super.onStart();
		
	}
	
	@Override
	protected void onStop() {
		super.onStop();
		try {
			mDbAdapter.close();
		} catch (NullPointerException e) {
			//DO NOTHING
		}
	}
	
	
	
	@Override
	protected void updateDialog(int id, Bundle args) {/*not used*/}
	/**
	 * Check if we have any matches
	 */
	private void checkMatch(){
		working = false;
		int numMatches = 0;
		int number = 0;
		for (int i = 0; i < matches.size(); i++) {
			if (matches.get(i)!= null && matches.get(i).score > Double.valueOf(prefs.getString("matchscore", null))) {
				number = i;
				numMatches++;
			}
		}
		if (numMatches == 0) {
			Toast.makeText(this, "I have no knowledge about this object.", Toast.LENGTH_LONG).show();
			startCameraActivity();
			imgButton.setClickable(true);
		}else if (numMatches > 1) {
			Toast.makeText(this, "There are several similar objects. Take a new picture of the object", Toast.LENGTH_LONG).show();
			startCameraActivity();
			imgButton.setClickable(true);
		}else{
			final long categoryId = memoryhelper.getModels(this).getIds().get(number);
			//category done before
			if (memoryhelper.checkCategoryDone(categoryId)) {
				categoryDoneHeader.setText("You have already completed: "+memoryhelper.getCategoryName(categoryId));
				String list = "";
				List<String> categoryList = memoryhelper.getAvailableCategories();
				//slide in done before
				for (String string : categoryList) {
					list += "- "+string+"\n";
				}
				availableCategoriesList.setText(list);
				mainLayout.setVisibility(LinearLayout.GONE);
				allreadyAnsweredLayout.setVisibility(LinearLayout.VISIBLE);
				allreadyAnsweredLayout.startAnimation(slideIn);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						allreadyAnsweredLayout.startAnimation(slideOut);
						allreadyAnsweredLayout.setVisibility(LinearLayout.GONE);
					}
				}, 4000);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						startCameraActivity();
						mainLayout.setVisibility(LinearLayout.VISIBLE);
					}
				}, 5000);
			}else{
				youChoseCategory.setText(memoryhelper.getCategoryName(categoryId));
				mainLayout.setVisibility(LinearLayout.GONE);
				newCategorySlideLayout.setVisibility(RelativeLayout.VISIBLE);
				newCategorySlideLayout.startAnimation(slideIn);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						newCategorySlideLayout.startAnimation(slideOut);
						newCategorySlideLayout.setVisibility(RelativeLayout.GONE);
					}
				}, 3000);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						Bundle bundle = new Bundle();
						bundle.putLong("assignmentSetId", categoryId);
						Intent theIntent = new Intent(InitializeCategoryActivity.this, QuizActivity.class);
						theIntent.putExtras(bundle);
						startActivity(theIntent);
					}
				}, 4000);
			}
		}
	}

	/**
	 * @param force Force reload of models
	 */
	private void fetchData(boolean force){
		memoryhelper = MemoryHelper.getMemoryHelper();
		if (memoryhelper.hasModels(this)) {
			if (force) {
				memoryhelper.removeModels(this);
			}else{
				return;
			}
		}
		//put categories into singleton
		Cursor categories = mDbAdapter.getCategories();
		Map<Long,String> cat = new HashMap<Long, String>();
		categories.moveToFirst();
		do {
			cat.put(categories.getLong(categories.getColumnIndexOrThrow("_id")),
					categories.getString(categories.getColumnIndexOrThrow("ctext")));
		} while (categories.moveToNext());
		memoryhelper.addCategories(this, cat);
		//put category models into singleton
		categories.close();
		Cursor categoryModels = mDbAdapter.getCategoryModels();
		if (!categoryModels.moveToFirst()) {
			Toast.makeText(this, "There are no category models. Stopping", Toast.LENGTH_LONG).show();
			categoryModels.close();
			return;
		}
		List<Long> modelId = new ArrayList<Long>();
		List<LocalModel<?>> models = new ArrayList<LocalModel<?>>();
		do {
			modelId.add(categoryModels.getLong(categoryModels.getColumnIndexOrThrow("cid")));
			models.add(FileStorage.loadLICOVmodel( 
							categoryModels.getBlob(categoryModels.getColumnIndexOrThrow("data"))));
		} while (categoryModels.moveToNext());
		
		memoryhelper.addModels(this, modelId, models);
		categoryModels.close();
	}

	/**
	 * Downloads latest database. 
	 * @param Force download database even if app already has been started before
	 */
	private void fetchDatabase(boolean force){
		
		//check if application has been started before and not finished
		if (!force &&  prefs.contains("dataBaseLoaded") && prefs.getBoolean("dataBaseLoaded", false) ) {
			mDbAdapter = new QuizSQLiteDBAdapter(this);
			mDbAdapter.open();
			return;
		}
		
		
		URL fileLocation = null;
		Exception processException = null;
		String text = null;
		Authenticator.setDefault(new UrlAuthentication( prefs.getString("databasehostuser", null), 
														prefs.getString("databasehostpass", null)));
		
		try {
			fileLocation = new URL( HTTP_URL_PROTOCOL, 
									prefs.getString("databasehost", null), 
									Integer.parseInt(prefs.getString("databasehostport", null)), 
									prefs.getString("databasename", null));
									
			mDbAdapter = new QuizSQLiteDBAdapter(this, fileLocation);
		} catch (MalformedURLException e) {
			text = "Malformed URL "+fileLocation.toString()+". "+e.getMessage();
			processException = e;
		} catch (Exception e) {
			text = "Error downloading database from "+fileLocation.getHost()+".\n"+e.getMessage();
			processException = e;
		}finally{
			if (processException != null) {		
				Log.e(TAG, text, processException);
				text = "Could not download database. Going to local backup";
				try {
					mDbAdapter = new QuizSQLiteDBAdapter(this, getAssets().open(ASSET_FILE));
				} catch (Exception e) {
					text = "Error loading database from assets. File: "+ASSET_FILE+".\n"+
				"Application has no database and can't operate. Exiting\n"+e.getMessage();
					Log.e(TAG, text);
					System.exit(1);
				}
			}else{
				text = "New database downloaded";
				Log.v(TAG, "Database downloaded from " +fileLocation.getHost());
			}
			//On for demonstration purposes
			Toast.makeText(this, text, Toast.LENGTH_LONG).show();
		}
		
		SharedPreferences.Editor editor = prefs.edit();
	    editor.putBoolean("dataBaseLoaded", true);
	    editor.commit();
		mDbAdapter.open();
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putBoolean("working", true);
	}
}
