package org.cyberlab;

import org.cyberlab.local.db.QuizSQLiteDBAdapter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.database.Cursor;
import android.os.Bundle;

/**
 * 
 * QuizEngine Helper class. Manages cursors from database and offers new questions in random order. When you are done
 * close the QuestionHelper
 * @author Mats-Gøran Karlsen
 *
 */
class QuestionHelper{
	private Map<Integer,List<Integer>> questionCollection;
	private List<Cursor> cursors;
	
	
	/**
	 * 
	 * 
	 * @param categoryId
	 * @param openedDatabase send in an already opened database
	 */
	public QuestionHelper(long categoryId, QuizSQLiteDBAdapter openedDatabase){
		questionCollection = new HashMap<Integer, List<Integer>>();
		cursors = new ArrayList<Cursor>();
		addQuestions(categoryId, openedDatabase);
	}
	
	/**
	 * Close Question Helper: closes all cursors associated with Helper
	 */
	public void close() {
		for (Cursor cursor : cursors) {
			cursor.close();
		}
	}

	/**
	 * Returns a bundle with a Text Question or a Recognition Question. If there are no more questions the bundle is empty;
	 * 
	 * @return Text Question: qid: question id, qtext: question text, aid: correct answer id
	 * 			Recognition Question: qid: question id, qtext: question text, _id:model id, mname:model name, data:model data
	 * 			Empty bundle if there are no more questions
	 */
	public Bundle nextQuestion(){
		
		Bundle results = new Bundle();
		
		int randomCursor = (int) ( Math.random() * cursors.size() );
		
		List<Integer> theList = questionCollection.get(randomCursor);
		//there are no more elements in the list aka there are no more cursors
		if (theList == null) {
			return results;
		}
		
		Integer theElement = theList.remove(theList.size()-1);
		Cursor theCursor = cursors.get(randomCursor);
		theCursor.moveToPosition(theElement);
		//check which cursor
		
		if (theCursor.getColumnIndex("aid") != -1) {
			results.putLong("qid", theCursor.getLong(theCursor.getColumnIndexOrThrow("qid")) );
			results.putString("qtext", theCursor.getString(theCursor.getColumnIndexOrThrow("qtext")) );
			results.putLong("aid", theCursor.getLong(theCursor.getColumnIndexOrThrow("aid")) );
		}
		
		if(theCursor.getColumnIndex("mname") != -1){
			results.putLong("qid", theCursor.getLong(theCursor.getColumnIndexOrThrow("qid")) );
			results.putString("qtext", theCursor.getString(theCursor.getColumnIndexOrThrow("qtext")) );
			results.putLong("_id", theCursor.getLong(theCursor.getColumnIndexOrThrow("_id")) );
			results.putString("mname", theCursor.getString(theCursor.getColumnIndexOrThrow("mname")) );
			results.putByteArray("data", theCursor.getBlob(theCursor.getColumnIndexOrThrow("data")) );
		}
		
		//check if we have gotten all elements from the cursor
		if (theList.isEmpty()) {
			questionCollection.remove(randomCursor);
			//update map keys to reflect changes in cursors length
			for (int i = randomCursor+1; i < cursors.size(); i++) {
				questionCollection.put(i-1, questionCollection.remove(i));
			}
			cursors.get(randomCursor).close();
			cursors.remove(randomCursor);
		}
		return results;
	}
	
	/**
	 * 
	 * Adds all category questions to a map
	 * @param categoryId
	 * @param openedDatabase
	 */
	private void addQuestions(long categoryId, QuizSQLiteDBAdapter openedDatabase){
		//Add TextQuestions
		Cursor textQuestions = openedDatabase.getTextQuestions(categoryId);
		if (textQuestions.getCount() != 0) {
			cursors.add(textQuestions);
			List<Integer> textQuestionOrder = new ArrayList<Integer>();
			for (int i = 0; i < textQuestions.getCount(); i++) {
				textQuestionOrder.add(i);
			}
			Collections.shuffle(textQuestionOrder);
			questionCollection.put(cursors.size()-1, textQuestionOrder);
		}else{
			textQuestions.close();
		}
		
		//Add RecognitionQuestions
		Cursor recognitionQuestions = openedDatabase.getRecognitionQuestionsAndModels(categoryId);
		if (recognitionQuestions.getCount() != 0) {
			cursors.add(recognitionQuestions);
			List<Integer> recognitionList = new ArrayList<Integer>();
			for (int i = 0; i < recognitionQuestions.getCount(); i++) {
				recognitionList.add(i);
			}
			Collections.shuffle(recognitionList);
			questionCollection.put(cursors.size()-1, recognitionList);
		}else{
			recognitionQuestions.close();
		}
	}
}
