package org.cyberlab;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cyberlab.local.activity.AbstractGeneralActivity;
import org.cyberlab.local.activity.task.CreateUnknownTask;
import org.cyberlab.local.activity.task.LicovMatchTask;
import org.cyberlab.animation.MyTranslateAnimation;
import org.cyberlab.animation.Rotate3dAnimation;
import org.cyberlab.local.recognition.LocalFeaturesExtractorFactory;
import org.cyberlab.local.recognition.LocalMatch;
import org.cyberlab.local.recognition.LocalModel;
import org.cyberlab.support.FileStorage;
import org.cyberlab.support.ImageOps;
import org.cyberlab.local.db.QuizSQLiteDBAdapter;

import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

/**
 * 
 * Runs the Quiz itself. Gets called from {@link InitializeCategoryActivity}. When there are no more categories this activity
 * slides in a demo summary and does nothing more. To rerun the demo either reinstall it or kill the application and then start it.
 * @author Mats-Gøran Karlsen
 *
 */
public class QuizActivity extends AbstractGeneralActivity<QuizActivity> implements OnClickListener{
	private SharedPreferences prefs;
	private Handler handler;
	private LinearLayout slidePickNewCategory, questionLayout, textQuestionButtonGroup;
	private RelativeLayout findObjectGroup;
	private TextView avaliableCategoriesList, questionText,  feedbackText;
	private ImageButton findObjectButton;
	private ImageButtonWithText[] textQuestionAnswerButtons = new ImageButtonWithText[4];
	private Map<Integer, Long> mapQuestionToButton;
	private ImageView scanbar;
	private MyTranslateAnimation scanAnimation;
	private Animation slideIn, slideOut;
	private MemoryHelper memoryhelper;
	private QuestionHelper questionhelper;
	private long categoryId;
	private Bundle currentQuestionBundle;
	private String textQuestionCorrectAnswer;
	private QuizSQLiteDBAdapter mDBAdabpter;
	private CreateUnknownTask createUnknown;
	private LicovMatchTask matchUnknown;
	private LocalModel<?> model;
	private LocalMatch<?> match;
	
	private boolean activityRestarted = false;
	
	@Override
	public void onClick(final View v) {
		switch (v.getId()) {
		//Object recognition question
		case R.id.findObjectButton:
			startCameraActivity();
			break;
		//Text Question
		default:
			if (currentQuestionBundle.getLong("aid") == mapQuestionToButton.get(v.getId())) {
				feedbackText.setText("Correct");
				feedbackText.setVisibility(TextView.VISIBLE);
				feedbackText.startAnimation(slideIn);
				((ImageButtonWithText)v).correct();
			    handler.postDelayed(new Runnable() { 
			         public void run() { 
			        	 rotate3D(feedbackText, 0, 90);
			        	 feedbackText.setVisibility(TextView.GONE);
			        	 for (ImageButtonWithText button : textQuestionAnswerButtons) {
								button.reset();
			        	 }
			        	 newQuestion();
			         } 
			    }, 2000); 
			}else{
				feedbackText.setText("Wrong");
				feedbackText.setVisibility(TextView.VISIBLE);
				feedbackText.startAnimation(slideIn);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						rotate3D(feedbackText, 0, 90);
						feedbackText.setVisibility(TextView.GONE);
					}
				}, 1000);
				((ImageButtonWithText)v).wrong();
				((ImageButtonWithText)v).setClickable(false);
				
			}
			break;
		}
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
	    super.onCreate(savedInstanceState);
	    
	    
	    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
	    categoryId = getIntent().getExtras().getLong("assignmentSetId");
	    memoryhelper = MemoryHelper.getMemoryHelper();
	    setTitle(memoryhelper.getCategoryName(categoryId));
	    
	    //fetch UI thread handler so we can queue operations on it
	    handler = new Handler(); 
	    //fetch and open database
	    mDBAdabpter = new QuizSQLiteDBAdapter(this);
	    mDBAdabpter.open();
	    setContentView(R.layout.quiz);
	    
	    //initialize category slide
	    slidePickNewCategory = (LinearLayout) findViewById(R.id.avaliableCategorySlide);
	    
	    avaliableCategoriesList = (TextView) findViewById(R.id.availableCategoriesList);
	    
	    //initialize textQuestion
	    questionLayout = (LinearLayout) findViewById(R.id.questionLayout);
	    questionLayout.setVisibility(LinearLayout.VISIBLE);
	    questionText = (TextView) findViewById(R.id.question);
	    feedbackText = (TextView) findViewById(R.id.feedback);
	    textQuestionAnswerButtons[0] = new ImageButtonWithText(this, R.id.textAnswer1Button);
	    textQuestionAnswerButtons[0].setOnClickListener(this);
	    textQuestionAnswerButtons[1] = new ImageButtonWithText(this, R.id.textAnswer2Button);
	    textQuestionAnswerButtons[1].setOnClickListener(this);
	    textQuestionAnswerButtons[2] = new ImageButtonWithText(this,R.id.textAnswer3Button);
	    textQuestionAnswerButtons[2].setOnClickListener(this);
	    textQuestionAnswerButtons[3] = new ImageButtonWithText(this, R.id.textAnswer4Button);
	    textQuestionAnswerButtons[3].setOnClickListener(this);
	    textQuestionButtonGroup = (LinearLayout) findViewById(R.id.buttonGroup);
	    
	    //initialize recognitionQuestion
	    findObjectGroup = (RelativeLayout) findViewById(R.id.findObjectGroup);
	    findObjectButton = (ImageButton) findViewById(R.id.findObjectButton);
	    scanbar = (ImageView) findViewById(R.id.scanbar);
	    findObjectButton.setOnClickListener(this);
	    
	    //initialize animations
	    slideIn = AnimationUtils.loadAnimation(this, R.anim.slidein);
	    slideOut = AnimationUtils.loadAnimation(this, R.anim.slideout);
	    
	    
	    prefs = PreferenceManager.getDefaultSharedPreferences(this);
	   
	    
	    if (savedInstanceState != null && savedInstanceState.containsKey("currentQuestion")) {
			currentQuestionBundle = savedInstanceState.getBundle("currentQuestion");
			questionhelper = (QuestionHelper) getLastNonConfigurationInstance();
			SharedPreferences.Editor editor = prefs.edit();
			editor.remove("retainQuestionHelper");
			editor.commit();
		}else{
		    //helperclass for organizing questions
			questionhelper = new QuestionHelper(categoryId, mDBAdabpter);
		}
	    
	    if (savedInstanceState != null && savedInstanceState.containsKey("activityRestarted")) {
			activityRestarted = true;
		}
	    newQuestion();
		
	}

	@Override
	public void threadFinished(int id) {
		if (id == 1) {
			//continue with matching
			matchUnknown = new LicovMatchTask(this, 2, createUnknown.getUnknown());
			matchUnknown.execute(model);
		}
		if (id == 2) {
			//retrieve match
			match = matchUnknown.getMatches()
								.get(0);
			checkMatch();
		}
	}

	@Override
	protected void cameraActivityResult() {
		Bitmap image = ImageOps.loadImage(mCameraFile.getPath(), 
											Integer.parseInt(prefs.getString("imagewidth", null)),
													Integer.parseInt(prefs.getString("imageheight", null)), true);
		findObjectButton.setImageBitmap(image);
		findObjectButton.setClickable(false);
		createUnknown = new CreateUnknownTask(this, 1, 
				LocalFeaturesExtractorFactory.socketExtractor(prefs.getString("detectorhost", null),
																Integer.parseInt(prefs.getString("detectorport", null)) ));
		scanAnimation = new MyTranslateAnimation(scanbar, Animation.RELATIVE_TO_PARENT, 0, Animation.RELATIVE_TO_PARENT, 0, 
				Animation.RELATIVE_TO_PARENT, 0, Animation.RELATIVE_TO_PARENT, 1.0f);
		createUnknown.execute(image);
		scanAnimation.startAnimation();
	}

	@Override
	protected void libraryActivityResult() {/*not used*/}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (!prefs.contains("retainQuestionHelper")) {
			questionhelper.close();
		}
		mDBAdabpter.close();
	}
	
	@Override
	protected void updateDialog(int id, Bundle args) {/*not used*/}

	private void checkMatch(){
		scanAnimation.stopAnimation();
		if (match != null && match.score > Double.valueOf(prefs.getString("matchscore", null))) {
			findObjectButton.setImageBitmap(match.drawBoundingBox());
			feedbackText.setText("You are correct! "+textQuestionCorrectAnswer);
			feedbackText.setVisibility(TextView.VISIBLE);
			feedbackText.startAnimation(slideIn);
			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					rotate3D(feedbackText,0,90);
					feedbackText.setVisibility(TextView.GONE);
				}
			}, 1000);
			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					newQuestion();
				}
			}, 2000);
			
		}else{
			feedbackText.setText("You answered incorrectly. Try again.");
			feedbackText.setVisibility(TextView.VISIBLE);
			feedbackText.startAnimation(slideIn);
			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					rotate3D(feedbackText,0,90);
					feedbackText.setVisibility(TextView.GONE);
				}
			}, 1000);
			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					startCameraActivity();
					findObjectButton.setClickable(true);
				}
			}, 2000);
		}
	}

	
	
	/**
	 * The QuestionEngine. Initializes new questions. Uses QuestionHelper
	 */
	private void newQuestion(){
		//prevents fetching of new question if snapping photo to answer question. This is added
	    //in case activity gets stopped while snapping photo.
	    
	    if (activityRestarted == false) {
	    	currentQuestionBundle = questionhelper.nextQuestion();
		}else{
			activityRestarted = false;
		}
		
		
		if (currentQuestionBundle.isEmpty()) {
			memoryhelper.categoryDone(categoryId);
			List<String> availableCategories = memoryhelper.getAvailableCategories();
			if (availableCategories.isEmpty()) {
				//Reset preference: database loaded so the demo can be rerun 
				SharedPreferences.Editor editor = prefs.edit();
			    if (prefs.contains("dataBaseLoaded")) {
			    	editor.remove("dataBaseLoaded");
			    	editor.commit();
			    }
			    //slide in thanks for playing
			    setTitle("Demo ended");
				final RelativeLayout credits = (RelativeLayout) findViewById(R.id.creditsLayout);
				questionLayout.setVisibility(LinearLayout.GONE);
				credits.setVisibility(RelativeLayout.VISIBLE);
				credits.startAnimation(slideIn);
				System.runFinalizersOnExit(true);
				
			}else{
				String list="";
				for (String category : availableCategories) {
					list += "- "+category+"\n";
				}
				avaliableCategoriesList.setText(list);
				questionLayout.setVisibility(LinearLayout.GONE);
				slidePickNewCategory.setVisibility(LinearLayout.VISIBLE);
				slidePickNewCategory.startAnimation(slideIn);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						slidePickNewCategory.startAnimation(slideOut);
						slidePickNewCategory.setVisibility(LinearLayout.GONE);
					}
				}, 4000);
				handler.postDelayed(new Runnable() {
					@Override
					public void run() {
						Intent intent = new Intent(QuizActivity.this, InitializeCategoryActivity.class);
						startActivity(intent);
						
					}
				}, 5000);
			}
		}else if (currentQuestionBundle.containsKey("aid")) {
			//text question
			long id = currentQuestionBundle.getLong("qid");
			questionText.setText(currentQuestionBundle.getString("qtext"));
			Cursor alternatives = mDBAdabpter.getTextQuestionAlternatives(id);
			alternatives.moveToFirst();
			ArrayList<Integer> indexes = new ArrayList<Integer>();
			for (int i = 0; i < textQuestionAnswerButtons.length; i++) {
				indexes.add(i);
			}
			Collections.shuffle(indexes);
			mapQuestionToButton = new HashMap<Integer, Long>();
			for (Integer integer : indexes) {
				textQuestionAnswerButtons[integer].setText(alternatives.getString(alternatives.getColumnIndexOrThrow("atext")));
				mapQuestionToButton.put(textQuestionAnswerButtons[integer].getId(), alternatives.getLong(alternatives.getColumnIndexOrThrow("_id")));
				alternatives.moveToNext();
			}
			findObjectGroup.setVisibility(LinearLayout.GONE);
			textQuestionButtonGroup.setVisibility(LinearLayout.VISIBLE);
			textQuestionButtonGroup.startAnimation(slideIn);
		}else if (currentQuestionBundle.containsKey("_id")) {
			//model question
			questionText.setText(currentQuestionBundle.getString("qtext"));
			model = FileStorage.loadLICOVmodel( currentQuestionBundle.getByteArray("data") );
			textQuestionCorrectAnswer = currentQuestionBundle.getString("mname");
			textQuestionButtonGroup.setVisibility(LinearLayout.GONE);
			findObjectGroup.setVisibility(LinearLayout.VISIBLE);
			findObjectGroup.startAnimation(slideIn);
		}
	}

	/**
	 * 
	 * Creates a 3D flip animation of the given view
	 * @param v the view in question
	 * @param start start angle in degrees
	 * @param finish end angle in degrees
	 */
	private void rotate3D(View v, int start, int finish){
			float centerX = v.getWidth() / 2.0f;
			float centerY = v.getHeight() / 2.0f;
	        // Create a new 3D rotation with the supplied parameter
	        Rotate3dAnimation rotation =
	                new Rotate3dAnimation(start, finish, centerX, centerY, 310.0f, true);
	        rotation.setDuration(500);
	        rotation.setFillAfter(true);
	        rotation.setInterpolator(new AccelerateInterpolator());
	        v.startAnimation(rotation);
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putBundle("currentQuestion", currentQuestionBundle);
		outState.putBoolean("activityRestarted", true);
		
	}

	@Override
	public Object onRetainNonConfigurationInstance() {
		SharedPreferences.Editor editor = prefs.edit();
		editor.putBoolean("retainQuestionHelper", true);
		editor.commit();
		return questionhelper;
	}

	
	
}