package org.cyberlab.tasks;

import org.cyberlab.InitializeCategoryActivity;
import org.cyberlab.QuizActivity;
import org.cyberlab.animation.MyTranslateAnimation;

import android.app.Activity;
import android.os.AsyncTask;
import android.view.animation.Animation;
import android.view.animation.TranslateAnimation;
import android.widget.ImageView;
import android.widget.Toast;

/**
 * 
 * Animates a scan line on Views during recognition operations. Used in {@link InitializeCategoryActivity} and {@link QuizActivity}.
 * @author Mats-Gøran Karlsen
 *
 */
public class ScanAnimationTask extends AsyncTask<Void, Void, Void>{

	protected final Activity act;
	private boolean running = true;
	private float startY = 0;
	private float endY = 1.0f;
	private ImageView bar;
	
	public ScanAnimationTask(Activity act, ImageView bar){
		this.act = act;
		this.bar = bar;
	}
	
	/**
	 * Ask the scan animation task to stop.
	 */
	public void stop(){
		running = !running;
	}

	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		Toast.makeText(act, "Searching...", Toast.LENGTH_LONG).show();
	}

	private void animate() {
		Runnable animation = new Runnable() {
			
			@Override
			public void run() {
				MyTranslateAnimation anim = new MyTranslateAnimation(bar, Animation.RELATIVE_TO_PARENT, 0, Animation.RELATIVE_TO_PARENT, 0, 
						Animation.RELATIVE_TO_PARENT, startY, Animation.RELATIVE_TO_PARENT, endY);
				anim.setRepeatCount(TranslateAnimation.INFINITE);
				anim.setRepeatMode(TranslateAnimation.REVERSE);
		        anim.setDuration(5000);
		        anim.setFillAfter(true);
		        bar.startAnimation(anim);
			}
		};
		act.runOnUiThread(animation);
		//float temp = startY;
		//startY = endY;
		//endY = temp;
	}
	
	@Override
	protected Void doInBackground(Void... params) {
		animate();
		while (running) {
			//animate();
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				//do nothing
			}
		}
		bar.getAnimation().cancel();
		return null;
	}
}
