package org.cyberlab;

import org.cyberlab.R;
import org.cyberlab.extendable.AbstractRootListActivity;

import android.app.Dialog;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Toast;

/**
 * 
 * Activity for showing end editing all Recognition assignments or Recognition assignments linked to an assignment set. Gets called from
 * {@link MainMenuActivity} and {@link SetListActivity}.
 * 
 * @author Mats-Gøran Karlsen
 *
 */
public class AssignmentListActivity extends AbstractRootListActivity {
	private static final String TAG = "AssignmentListActivity";
	
	private final static int ASSIGNMENT_INFO = 8888;
	private final static int NEW_ANSWER = 8889;
	
	@Override
	protected void fillData() {
		Cursor assignmentCursor = mDbAdapter.getModelAssignments(filterId);
		startManagingCursor(assignmentCursor);
		String[] from = new String[] {"qtext","ctext"};
		int[] to = new int[] {R.id.row_head,R.id.row_sub};
		SimpleCursorAdapter assignments = new SimpleCursorAdapter(this, R.layout.listrow, assignmentCursor, from, to);
		setListAdapter(assignments);
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		Bundle bundle = new Bundle();
		bundle.putLong("assignmentId", id);
		bundle.putString("assignmentName", ((TextView) v.findViewById(R.id.row_head)).getText().toString() );
		Cursor modelCursor = mDbAdapter.getAssignmentModel(id, false);
		modelCursor.moveToFirst();
		bundle.putLong("objectId",  modelCursor.getLong(modelCursor.getColumnIndexOrThrow("_id")) );
		bundle.putString("objectName" , modelCursor.getString( modelCursor.getColumnIndexOrThrow("mname") ) );
		modelCursor.close();
		showDialog(ASSIGNMENT_INFO, bundle);
	}

	@Override
	protected Dialog onCreateDialog(int id, final Bundle args) {
		if (id == ASSIGNMENT_INFO) {
			final Dialog editDialog = new Dialog(AssignmentListActivity.this);
			editDialog.setContentView(R.layout.assignmentedit);
			editDialog.setTitle(args.getString("assignmentName"));
			TextView objectname = (TextView) editDialog.findViewById(R.id.objectText);
			objectname.setText(args.getString("objectName"));
			Button edit = (Button) editDialog.findViewById(R.id.edit_assignment);
			edit.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					Intent intent = new Intent(AssignmentListActivity.this, FromAssignmentModelListActivity.class);
					intent.putExtras(args);
					removeDialog(ASSIGNMENT_INFO);
					startActivityForResult(intent, ASSIGNMENT_INFO);
				}
			});
			Button close = (Button) editDialog.findViewById(R.id.close_editassignment);
			close.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(ASSIGNMENT_INFO);
				}
			});
			return editDialog;
		}
		if (id == NEW_ANSWER) {
			final Dialog confirmDialog = new Dialog(AssignmentListActivity.this);
			confirmDialog.setContentView(R.layout.assignmenteditconfirm);
			confirmDialog.setTitle(args.getString("assignmentName"));
			String confirmText = getString(R.string.confirm_headText)+" "+args.getString("assignmentName")
					+" "+getString(R.string.confirm_tailText)+" "+args.getString("objectName");
			TextView text = (TextView) confirmDialog.findViewById(R.id.confirmText);
			text.setText(confirmText);
			Button cancel = (Button) confirmDialog.findViewById(R.id.assignmentconfirmcancel);
			cancel.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(NEW_ANSWER);
				}
			});
			Button confirm = (Button) confirmDialog.findViewById(R.id.assignmentconfirmok);
			confirm.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					editAnswer(args);
					removeDialog(NEW_ANSWER);
				}
			});
			
			return confirmDialog;
		}		
		return null;
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == ASSIGNMENT_INFO) {
			if (resultCode == RESULT_OK) {
				showDialog(NEW_ANSWER, data.getExtras());
			}
		}
	}

	/**
	 * Edit answer to assignment in database
	 * @param data Bundle containing necessary information to commit changes in database.
	 */
	private void editAnswer(Bundle data){
		long result = mDbAdapter.changeAssignmentModel(data.getLong("assignmentId"), data.getLong("objectId"));
		if ( result > 0) {
			Toast.makeText(this, data.getString("assignmentName")+" was successfully updated", Toast.LENGTH_LONG).show();
		}else{
			Log.e(TAG, "Error updating assignment "+data.getString("assignmentName")+" with id: "
		+String.valueOf(data.getLong("assignmentId"))+". The model id was "+String.valueOf(data.getLong("objectId"))+".");
			Toast.makeText(this, data.getString("assignmentName")+" update failed.", Toast.LENGTH_LONG).show();
		}
		

	}

}
