package org.cyberlab;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cyberlab.CustomImageView.RecognitionData;
import org.cyberlab.local.activity.AbstractCreateModelActivity;
import org.cyberlab.local.activity.task.CreateLicovModelTask;
import org.cyberlab.local.activity.task.AbstractCreateModelTask;
import org.cyberlab.local.activity.task.CreateUnknownTask;
import org.cyberlab.local.activity.task.LicovMatchTask;
import org.cyberlab.local.activity.task.TaskManager;
import org.cyberlab.local.activity.task.AbstractWorkerTask;
import org.cyberlab.local.db.QuizSQLiteDBAdapter;
import org.cyberlab.local.recognition.LocalFeaturesExtractorFactory;
import org.cyberlab.local.recognition.LICOV.LICOVModel;
import org.cyberlab.support.ImageOps;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;

/**
 * @author matsgora
 *
 * Activity for creating LICOV models. Partially support for device rotation and uses tasks for long running operations.
 * See Buglist for information on what needs to be fixed in order to get device rotation fully functional.
 *
 */
public class CreateLicovModelActivity extends AbstractCreateModelActivity<LICOVModel> {
	private static final int DIALOG = 1234;
	private static final int DIALOG_RESULTS = 1235;
	private SharedPreferences prefs;
	private Bundle intentBundle;
	private int imageHeight, imageWidth;
	private CustomImageView imgv1, imgv2;
	private int viewUpdated;
	private Button but;
	private CustomProgressDialog progress;
	private CreateUnknownTask featTask1, featTask2;
	private LicovMatchTask match1, match2;
	/**
	 * @author matsgora
	 * Enum to keep track of activity threads
	 */
	private enum threadID {
		MODEL(0), MATCH1(1), MATCH2(2), UNKNOWN1(3), UNKNOWN2(4);

		private int code;
		
		private threadID(int c){
			code = c;
		}
		
		public int getCode(){
			return code;
		}
	}

	private OnClickListener mImageListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			startCameraActivity();
			viewUpdated = ((CustomImageView) v).getId();
		}
	};
	
	/**
	 * Activity message handler. This is used by threads to update the progress dialog.
	 * @return Message handler
	 */
	public Handler getHandler() {
		return mHandler;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.modelcreatemenu, menu);
		return true;
	}

	@Override
	public Object onRetainNonConfigurationInstance() {
		final Map<String, RecognitionData> data = saveRecognitionData();
		return data;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent = new Intent(CreateLicovModelActivity.this, CreatorHelpAcvtivity.class);
		startActivity(intent);
		return true;
	}

	/**
	 * Threads notify the activity when they finish. This allows the activity to update itself to thread events
	 * 
	 * @param id Thread id
	 */
	public void threadFinished(int id){
		if (id == threadID.MODEL.getCode()) {
			super.setModel(mModelTask.getModel());
			imgv1.setUnknown(featTask1.getUnknown());
			imgv1.drawKeypoints();
			imgv2.setUnknown(featTask2.getUnknown());
			imgv2.drawKeypoints();
			if (featTask1.getUnknown() != null || featTask1.getUnknown() != null ) {
				matchModel();
			}
		}
		
		if (id == threadID.UNKNOWN1.getCode()) {
			imgv1.setUnknown(featTask1.getUnknown());
			if (featTask2.getStatus() == AsyncTask.Status.FINISHED) {
				buildModel();
			}
		}
		
		if (id == threadID.UNKNOWN2.getCode()) {
			imgv2.setUnknown(featTask2.getUnknown());
			if (featTask1.getStatus() == AsyncTask.Status.FINISHED) {
				buildModel();
			}
		}
		
		if (id == threadID.MATCH1.getCode()) {
			imgv1.drawMatchBoundingBox( match1.getMatches().get(0) );
		}
		if (id == threadID.MATCH2.getCode()) {
			imgv2.drawMatchBoundingBox( match2.getMatches().get(0) );
		}
	}

	@Override
	protected void cameraActivityResult() {
		if (viewUpdated == imgv1.getId()) {
			imgv1.setImageBitmap(ImageOps.loadImage(mCameraFile.getPath(),
					imageWidth, imageHeight, true));
			updateButton();
		} else {
			imgv2.setImageBitmap(ImageOps.loadImage(mCameraFile.getPath(),
					imageWidth, imageHeight, true));
			updateButton();
		}
	}

	@Override
	protected void libraryActivityResult() {/*not used*/}

	@SuppressWarnings("unchecked")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		prefs = PreferenceManager.getDefaultSharedPreferences(this);
		imageWidth = Integer.parseInt(prefs.getString("imagewidth", null));
		imageHeight = Integer.parseInt(prefs.getString("imagewidth", null));
		setContentView(R.layout.modelbuild);
		imgv1 = (CustomImageView) findViewById(R.id.imageView1);
		imgv1.setOnClickListener(mImageListener);
		imgv2 = (CustomImageView) findViewById(R.id.imageView2);
		imgv2.setOnClickListener(mImageListener);
		but = (Button) findViewById(R.id.button1);
		but.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				preProcessImages();
			}
		});
		//load saved nonconfiginstances
		final Map<String, RecognitionData> data = (Map<String, RecognitionData>) getLastNonConfigurationInstance();
		if (data != null) {
			if (data.containsKey("imgv1")) {
				imgv1.setRecocnitionData(data.get("imgv1"));
			}

			if (data.containsKey("imgv2")) {
				imgv2.setRecocnitionData(data.get("imgv2"));
			}
		}
		updateButton();
		intentBundle = getIntent().getExtras();
		if (!prefs.contains("runbefore")) {
			SharedPreferences.Editor editor = prefs.edit();
			editor.putBoolean("runbefore", true);
			editor.commit();
			startActivity(new Intent(CreateLicovModelActivity.this, CreatorHelpAcvtivity.class));
		}
		
		//load saved instances
		if (savedInstanceState != null) {
			if (savedInstanceState.containsKey("dialog_total")
					&& savedInstanceState.containsKey("dialog_received")
					&& savedInstanceState.containsKey("dialog_text")) {
				showDialog(DIALOG, savedInstanceState);
			}
			
			if (savedInstanceState.containsKey("viewUpdated")) {
				viewUpdated = savedInstanceState.getInt("viewUpdated");
			}
		}
		List<AbstractWorkerTask<?, ?>> tasks = TaskManager.getTaskManager().attach(this);
		if (tasks != null) {
			for (AbstractWorkerTask<?, ?> workerTask : tasks) {
				if (workerTask.getId() == threadID.UNKNOWN1.getCode()) {
					featTask1 = (CreateUnknownTask) workerTask;
				}
				if (workerTask.getId() == threadID.UNKNOWN2.getCode()) {
					featTask2 = (CreateUnknownTask) workerTask;
				}
				if (workerTask.getId() == threadID.MODEL.getCode()) {
					mModelTask = (AbstractCreateModelTask<LICOVModel>) workerTask;
				}	
				if (workerTask.getId() == threadID.MATCH1.getCode()) {
					match1 =  (LicovMatchTask) workerTask;
				}
				if (workerTask.getId() == threadID.MATCH2.getCode()) {
					match2 =  (LicovMatchTask) workerTask;
				}	
			}
		}
		if (match1 != null && match1.getStatus() == AsyncTask.Status.FINISHED) {
			imgv1.drawMatchBoundingBox(match1.getMatches().get(0));
		}
		if (match2 != null && match2.getStatus() == AsyncTask.Status.FINISHED) {
			imgv2.drawMatchBoundingBox(match2.getMatches().get(0));
		}
	}
	
	@Override
	protected Dialog onCreateDialog(int id, Bundle args) {
		switch (id) {
		case DIALOG:
			progress = new CustomProgressDialog(this, args.getInt("threads"));
			progress.setCanceledOnTouchOutside(false);
			progress.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
			progress.updateDialogMessage(this, args.getString("dialog_text") );
			if (args.containsKey("dialog_total")) {
				progress.setTotal(args.getInt("dialog_total"));
				progress.setReceived(args.getInt("dialog_received"));
				
			}else{
				progress.setProgress(0);
			}
			return progress;
		case DIALOG_RESULTS:
			final Dialog resultsDialog = new Dialog(CreateLicovModelActivity.this);
			resultsDialog.setContentView(R.layout.modelresults);
			resultsDialog.setTitle(getString(R.string.model_summary));	
			TextView score1 = (TextView) resultsDialog.findViewById(R.id.match_score1);
			score1.setText(getString(R.string.score1)+match1.getMatches().get(0).score);
			TextView score2 = (TextView) resultsDialog.findViewById(R.id.match_score2);
			score2.setText(getString(R.string.score_2)+match2.getMatches().get(0).score);
			TextView numKp = (TextView) resultsDialog.findViewById(R.id.numKp);
			numKp.setText(CreateLicovModelActivity.this.getString(R.string.modelresults)+((LICOVModel)mModel).size());
			ProgressBar bar = (ProgressBar) resultsDialog.findViewById(R.id.progressBar1);
			int quality = (int) (((LICOVModel) mModel).size() / (double)32 * 100) ;
			bar.setProgress(quality);
			Button confirm = (Button) resultsDialog.findViewById(R.id.model_confirm);
			confirm.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(DIALOG_RESULTS);
					writeModel();
				}
			});
			Button cancel = (Button) resultsDialog.findViewById(R.id.model_cancel);
			cancel.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(DIALOG_RESULTS);
				}
			});
			return resultsDialog;
		default:
			return null;
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		if (featTask1 != null) {
			TaskManager.getTaskManager().detach(this, featTask1);
		}
		
		if (featTask2 != null) {
			TaskManager.getTaskManager().detach(this, featTask2);
		}
		if (mModelTask != null
				&& mModelTask.getStatus() == AsyncTask.Status.RUNNING) {
			TaskManager.getTaskManager().detach(this, mModelTask);
		}
		if (match1 != null) {
			TaskManager.getTaskManager().detach(this, match1);
		}
		if (match2 != null) {
			TaskManager.getTaskManager().detach(this, match2);
		}
		if (progress != null && progress.isShowing()) {
			saveProgressDialog(outState);
			removeDialog(DIALOG);
		}
		outState.putInt("viewUpdated", viewUpdated);
	}

	/**
	 * Update a progress dialog 
	 * 
	 * @param id of dialog
	 * @param args Message to dialog
	 */
	protected void updateDialog(int id, Bundle args){
		switch (id) {
		case DIALOG:
			if (args.containsKey("thread_progress")) {
				progress.setProgress(args.getInt("thread_progress"));
			}
			if (args.containsKey("dialog_text")) {
				progress.updateDialogMessage(this, args.getString("dialog_text"));
			}
			if (progress.getProgress() >= 100) {
				removeDialog(DIALOG);
				showDialog(DIALOG_RESULTS);
			}
			break;
		default:
			break;
		}
	}
	
	/**
	 * After the two Unknown objects are created the build model task is executed 
	 */
	private void buildModel(){
		mModelTask = new CreateLicovModelTask(this, threadID.MODEL.getCode());
		mModelTask.execute(imgv1.getRecognitionData().obj, imgv2.getRecognitionData().obj);
		Bundle bundle = new Bundle();
		bundle.putString("dialog_text", "Creating model...");
		updateDialog( DIALOG, bundle );
	}

	/**
	 * Matches the newly created model onto the input images. This gives and 
	 * indication on the quality of the model in relation to the input images. 
	 * 
	 */
	private void matchModel(){
		match1 = new LicovMatchTask(this, threadID.MATCH1.getCode(), imgv1.getUnknown());
		match2 = new LicovMatchTask(this, threadID.MATCH2.getCode(), imgv2.getUnknown());
		match1.execute(mModel);
		match2.execute(mModel);
		Bundle bundle = new Bundle();
		bundle.putString("dialog_text", "Matching model onto images...");
		updateDialog( DIALOG, bundle );
	}

	/**
	 * Creates unknown objects.
	 * 
	 */
	private void preProcessImages() {
		if (mExtractor == null) {
			mExtractor = LocalFeaturesExtractorFactory.socketExtractor(
															prefs.getString("detectorhost", null), 
															Integer.parseInt(prefs.getString("detectorport", null)));
		}
		Bundle bundle = new Bundle();
		bundle.putInt("threads", 5);
		bundle.putString("dialog_text", "Detecting features...");
		featTask1 = new CreateUnknownTask(this, threadID.UNKNOWN1.getCode(), mExtractor);
		featTask2 = new CreateUnknownTask(this, threadID.UNKNOWN2.getCode(), mExtractor);
		featTask1.execute(imgv1.getImageBitmap());
		featTask2.execute(imgv2.getImageBitmap());
		showDialog((DIALOG), bundle);	
	}

	/**
	 * Save state of dialog if system restarts activity due to device rotation.
	 * 
	 * @param outState Bundle used by system to save activity state
	 */
	private void saveProgressDialog(Bundle outState){
		outState.putString("dialog_text", (String) progress.getMessage());
		outState.putInt("dialog_total", progress.getTotal());
		outState.putInt("dialog_received", progress.getReceived());
	}
	
	/**
	 * Collect recognition data if activity gets restarted. This gets called in
	 * onRetainConfigurationData
	 * @return Map containing recognition data
	 */
	private Map<String, RecognitionData> saveRecognitionData() {
		Map<String, RecognitionData> imagedata = new HashMap<String, RecognitionData>();
		if (imgv1.getRecognitionData() != null) {
			imagedata.put("imgv1", imgv1.getRecognitionData());
		}
		if (imgv2.getRecognitionData() != null) {
			imagedata.put("imgv2", imgv2.getRecognitionData());
		}
		return imagedata;
	}

	/**
	 * Activates create model button. Button is activated when both CustomImageViews have bitmaps.
	 */
	private void updateButton(){
		if ( (imgv1.getImageBitmap() != null) && (imgv2.getImageBitmap() != null) ) {
			but.setClickable(true);
			but.setTextColor(Color.BLACK);			
		}else{
			but.setClickable(true);
			but.setTextColor(Color.GRAY);
		}
		
	}
	
	/**
	 * Writes model to database and finishes activity
	 */
	private void writeModel(){
		QuizSQLiteDBAdapter mDbAdapter = new QuizSQLiteDBAdapter(this);
		mDbAdapter.open();
		if (intentBundle != null && intentBundle.containsKey("objectId")) {
			if (mDbAdapter.editModel(intentBundle.getLong("objectId"), null, mModel) == 1) {
				intentBundle.putBoolean("editResult", true);
			}
			else {
				intentBundle.putBoolean("editResult", false);
			}
		}else{
			long id = mDbAdapter.addModel(mModel);
			if (intentBundle == null) {
				intentBundle = new Bundle();
			}
			intentBundle.putLong("objectId", id);
		}
		mDbAdapter.close();
		Intent intent = new Intent();
		intent.putExtras(intentBundle);
		setResult(RESULT_OK, intent);
		finish();
	}
}
