package org.cyberlab;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;

/**
 * 
 * Progress dialog for correctly displaying progress when multiple parallel threads are posting progress. Class
 * has a set of convenience functions not available in superclass for getting state of dialog. 
 * Rationale: Save state of progress dialog if activity is restarted.
 *
 * @author Mats-Gøran Karlsen
 */
public class CustomProgressDialog extends ProgressDialog {
	private int total;
	private int received = 0;
	private CharSequence text;
	
	/**
	 * @param context Activity context
	 * @param numThreads Number of concurrent threads posting to dialog
	 */
	public CustomProgressDialog(Context context, int numThreads) {
		super(context);
		
		this.total = numThreads*100;
	}
	
	public CharSequence getMessage(){
		return this.text;
	}

	public int getReceived() {
		return received;
	}

	public int getTotal() {
		return total;
	}

	@Override
	public void setMessage(CharSequence message) {
		this.text = message;
		super.setMessage(message);
	}

	@Override
	public void setProgress(int chunk) {
		received += chunk;
		super.setProgress( (int) (  ( received/ (double) total )  * 100 ) );
	}

	public void setReceived(int received) {
		this.received = received;
		super.setProgress( (int) (  ( received/ (double) total )  * 100 ) );
	}

	public void setTotal(int total) {
		this.total = total;
	}

	/**
	 * 
	 * Updates the dialog message
	 * 
	 * @param act Current activity of the dialog
	 * @param message New message
	 */
	protected void updateDialogMessage(final Activity act, final CharSequence message){
		Runnable changeMessage = new Runnable() {
			@Override
			public void run() {
				setMessage(message);
			}
		};
		act.runOnUiThread(changeMessage);
	}

}
