package org.cyberlab;

import org.cyberlab.R;
import org.cyberlab.extendable.AbstractModelListActivity;

import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * 
 * Model list activity that gets called from  {@link AssignmentListActivity}. 
 * Rationale: Specific menu functionality to this case: changing the answer to a Recognition Question by either picking
 * an existing model or adding an new model to the database.
 * @author Mats-Gøran Karlsen
 *
 */
public class FromAssignmentModelListActivity extends AbstractModelListActivity  {

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == EDIT_OBJECT) {
			if (resultCode == RESULT_OK) {
				Bundle result = data.getExtras();
				String text;
				if (result.getBoolean("editResult")) {
					text = result.getString("objectName")+" modified"; 
				}else {
					text = result.getString("objectName")+": modification failed";
				}
				
				Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
				return;
				
			}
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		ListView lv = getListView();
		lv.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
			@Override
			public boolean onItemLongClick(AdapterView<?> av, View v,
					int position, long id) {
				Bundle bundle = new Bundle();
				bundle.putLong("objectId",id);
				bundle.putString("objectName", ((TextView)v.findViewById(R.id.row_head)).getText().toString() );
				showDialog(EDIT_OBJECT, bundle);
				return false;
			}
		});
	}
	
	@Override
	protected Dialog onCreateDialog(int id, final Bundle args) {
		
		if (id == EDIT_OBJECT) {
			final Dialog editModelDialog = new Dialog(FromAssignmentModelListActivity.this);
			editModelDialog.setContentView(R.layout.assignmentedit);
			editModelDialog.setTitle(args.getString("objectName"));
			TextView label = (TextView)editModelDialog.findViewById(R.id.answerLabel);
			label.setText("");
			TextView objText = (TextView)editModelDialog.findViewById(R.id.objectText);
			objText.setText("");
			Button editButton = (Button) editModelDialog.findViewById(R.id.edit_assignment);
			editButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(EDIT_OBJECT);
					Intent intent = new Intent(FromAssignmentModelListActivity.this, CreateLicovModelActivity.class);
					intent.putExtras(args);
					startActivityForResult(intent, EDIT_OBJECT);
				}
			});
			Button dismissButton = (Button) editModelDialog.findViewById(R.id.close_editassignment);
			dismissButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(EDIT_OBJECT);
				}
			});
			
			return editModelDialog;
		}
		
		return super.onCreateDialog(id,args);
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		intentExtras.putLong("objectId", id);
		intentExtras.putString("objectName", ((TextView)v.findViewById(R.id.row_head)).getText().toString() );
		Intent intent = new Intent();
		intent.putExtras(intentExtras);
		setResult(RESULT_OK, intent);
		finish();
	}
	
}
