package org.cyberlab;

import org.cyberlab.R;
import org.cyberlab.extendable.AbstractModelListActivity;

import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


/**
 *
 * Activity showing all objects stored in database. Allows edit/add/deletion of objects.
 * This activity gets called from {@link MainMenuActivity}.
 *
 * @author Mats-Gøran Karlsen
 */
public class ModelListActivity extends AbstractModelListActivity {

	//after add object
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == EDIT_OBJECT) {
			if (resultCode == RESULT_OK) {
				Bundle result = data.getExtras();
				String text;
				if (result.getBoolean("editResult")) {
					text = result.getString("objectName")+" modified"; 
				}else {
					text = result.getString("objectName")+": modification failed";
				}
				
				Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
				return;
			}
		}
	}

	@Override
	protected Dialog onCreateDialog(int id, final Bundle args) {
		if (id == EDIT_OBJECT) {
			final Dialog editDialog = new Dialog(ModelListActivity.this);
			editDialog.setContentView(R.layout.modeledit);
			editDialog.setTitle(R.string.modify_object_title);
			TextView objName = (TextView) editDialog.findViewById(R.id.textObjectName);
			objName.setText(args.getString("objectName"));
			Button edit = (Button) editDialog.findViewById(R.id.edit_object);
			edit.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(EDIT_OBJECT);
					Intent intent = new Intent(ModelListActivity.this, CreateLicovModelActivity.class);
					intent.putExtras(args);
					startActivityForResult(intent, EDIT_OBJECT);
				}
			});
			Button delete = (Button) editDialog.findViewById(R.id.delete_object);
			delete.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(EDIT_OBJECT);
					deleteObject(args);
				}
			});
			Button close = (Button) editDialog.findViewById(R.id.close_dialog);
			close.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					removeDialog(EDIT_OBJECT);
				}
			});
			return editDialog;
			
		}
		return super.onCreateDialog(id, args);
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		Bundle bundle = new Bundle();
		//TODO add info
		bundle.putLong("objectId", id);
		bundle.putString("objectName", ((TextView) v.findViewById(R.id.row_head)).getText().toString() );
		showDialog(EDIT_OBJECT, bundle);
	}

	
	/**
	 * 
	 * Removes an object from the database if it is not connected to an assignment.
	 * 
	 * @param data bundle containing information needed for database update
	 */
	private void deleteObject(Bundle data){
		String text;
		if (mDbAdapter.deleteModel(data.getLong("objectId")) == 1) {
			text = data.getString("objectName")+" deleted";
		}else{
			text = "Deletion failed: "+data.getString("objectName")+" is linked to an assignment or category!";
		}
		Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
		fillData();
	}
	
}
