package org.cyberlab.extendable;
import org.cyberlab.CreateLicovModelActivity;
import org.cyberlab.FromAssignmentModelListActivity;
import org.cyberlab.ModelListActivity;
import org.cyberlab.R;

import android.app.Dialog;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.SimpleCursorAdapter;


/**
 * 
 * Common methods for {@link ModelListActivity} and {@link FromAssignmentModelListActivity}.
 * @author Mats-Gøran Karlsen
 *
 */
public abstract class AbstractModelListActivity extends AbstractRootListActivity {

	protected static final int EDIT_OBJECT = 1337;
	protected int addobjectid;
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		boolean result = super.onCreateOptionsMenu(menu);
		addobjectid = menu.size()+1;
		menu.add(0, addobjectid, 0, R.string.add_object);
		return result;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (item.getItemId() == addobjectid) {
			Intent intent = new Intent(AbstractModelListActivity.this, CreateLicovModelActivity.class);
			startActivityForResult(intent, addobjectid);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void fillData() {
		Cursor modelsCursor = mDbAdapter.getModels(false);
		startManagingCursor(modelsCursor);
		String[] from = new String[] {"mname","qtext"};
		int[] to = new int[] {R.id.row_head,R.id.row_sub};
		SimpleCursorAdapter models = new SimpleCursorAdapter(this, R.layout.listrow, modelsCursor, from, to);
		setListAdapter(models);	
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == addobjectid) {
			if (resultCode == RESULT_OK) {
				showDialog(addobjectid, data.getExtras());
				return;
			}
		}
	}

	@Override
	protected Dialog onCreateDialog(int id, final Bundle args) {
		if (id == addobjectid) {
			final Dialog nameDialog = new Dialog(AbstractModelListActivity.this);
			nameDialog.setContentView(R.layout.modelname);
			nameDialog.setTitle(R.string.namemodel_dialog_title);
			final EditText newText = (EditText) nameDialog.findViewById(R.id.modelname_edit);
			final Button confirm = (Button) nameDialog.findViewById(R.id.model_save);
			newText.addTextChangedListener(new TextWatcher() {
				@Override
				public void onTextChanged(CharSequence s, int start, int before, int count) {/*not used*/}
				@Override
				public void beforeTextChanged(CharSequence s, int start, int count,
						int after) {/*not used*/}
				@Override
				public void afterTextChanged(Editable s) {
					confirm.setClickable(true);
				}
			});
			confirm.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					args.putString("objectName", newText.getText().toString().trim());
					addObject(args);
					removeDialog(addobjectid);
				}
			});
			confirm.setClickable(false);
			return nameDialog;
		}
		return null;
	}
	
	/**
	 * Adds a new object to the database
	 * @param data bundle containing information needed for database update
	 */
	private void addObject(Bundle data){
		mDbAdapter.editModel(data.getLong("objectId"), data.getString("objectName"), null);
		fillData();
	}

}
