package org.cyberlab.extendable;
import org.cyberlab.AssignmentListActivity;
import org.cyberlab.MainMenuActivity;
import org.cyberlab.R;
import org.cyberlab.SetListActivity;
import org.cyberlab.local.db.QuizSQLiteDBAdapter;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;


/**
 * 
 * Common methods for {@link SetListActivity}, {@link AssignmentListActivity}, {@link AbstractModelListActivity}.
 * @author Mats-Gøran Karlsen
 * 
 * 
 */
public abstract class AbstractRootListActivity extends ListActivity {

	//TODO add database
	protected QuizSQLiteDBAdapter mDbAdapter;
	
	/**
	 * Subclasses use this variable to filter the database query. Ex if SetListActivity sends filterId to AssignmentListActivity, 
	 * AssignmentListActivity queries the database for items only containing this id. 
	 * If filterId is not set all assignment items can be retrieved.
	 */
	protected int filterId;
	protected Bundle intentExtras;
	
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.basemenuoptions, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.backtomain:
			Intent intent = new Intent(this, MainMenuActivity.class);
			intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
			intent.setFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
			startActivity(intent);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		setContentView(R.layout.listitems);
		mDbAdapter = new QuizSQLiteDBAdapter(this);
		mDbAdapter.open();
		if (getIntent().getExtras() != null) {
			intentExtras = getIntent().getExtras();
			if (getIntent().getExtras().containsKey("filterId")) {
				filterId = getIntent().getExtras().getInt("filterId");
			}
		}else{
			filterId = -1;
		}
		fillData();
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		mDbAdapter.close();
	}

	/**
	 *  Method for populating list at initiation and update.
	 */
	protected abstract void fillData();

	
}
